package Business::Westpac;

=head1 NAME

Business::Westpac - Top level namespace for the Business::Westpac
set of modules

=head1 VERSION

0.01

=head1 DESCRIPTION

Business::Westpac is a set of libraries for parsing/generating files
for use with Westpac bank as documented at
L<https://paymentsplus.westpac.com.au/docs/>

=head1 Do Not Use This Module Directly

You should go to the relevant module for the file format you want to
work with.

=cut

$Business::Westpac::VERSION = '0.01';

=head1 SEE ALSO

L<Business::Westpac::PaymentsPlus::Australian::Payment::Import::File>

=head1 AUTHOR

Lee Johnson - C<leejo@cpan.org>

=head1 LICENSE

This library is free software; you can redistribute it and/or modify it under
the same terms as Perl itself. If you would like to contribute documentation,
features, bug fixes, or anything else then please raise an issue / pull request:

    https://github.com/leejo/business-westpac

=cut

1;
